@@ -1,6 +1,6 @@
 """liquidctl drivers for sixth generation Asetek liquid coolers.
 
-Copyright (C) 2020–2021  Andrew Robertson, Jonas Malaco and contributors
+Copyright (C) 2020–2022  Andrew Robertson, Jonas Malaco and contributors
 
 SPDX-License-Identifier: GPL-3.0-or-later
 """
@@ -79,13 +79,13 @@ def _quoted(*names):
 
 # we inherit from _Base690Lc to reuse its implementation of connect
 # and disconnect, that emulates the stock SiUSBXp driver on Windows
-class CorsairAsetekProDriver(_Base690Lc):
+class HydroPro(_Base690Lc):
     """liquidctl driver for Corsair-branded sixth generation Asetek coolers."""
 
     SUPPORTED_DEVICES = [
-        (0x1b1c, 0x0c12, None, 'Corsair Hydro H150i Pro (experimental)', {'fan_count': 3}),
-        (0x1b1c, 0x0c13, None, 'Corsair Hydro H115i Pro (experimental)', {'fan_count': 2}),
-        (0x1b1c, 0x0c15, None, 'Corsair Hydro H100i Pro (experimental)', {'fan_count': 2})
+        (0x1b1c, 0x0c12, None, 'Corsair Hydro H150i Pro', {'fan_count': 3}),
+        (0x1b1c, 0x0c13, None, 'Corsair Hydro H115i Pro', {'fan_count': 2}),
+        (0x1b1c, 0x0c15, None, 'Corsair Hydro H100i Pro', {'fan_count': 2})
     ]
 
     def __init__(self, device, description, fan_count, **kwargs):
@@ -260,3 +260,7 @@ class CorsairAsetekProDriver(_Base690Lc):
     @classmethod
     def probe(cls, handle, **kwargs):
         return super().probe(handle, **kwargs)
+
+
+# backward compatibility
+CorsairAsetekProDriver = HydroPro
