@@ -1,6 +1,6 @@
 """liquidctl drivers for DDR4 memory.
 
-Copyright (C) 2020–2021  Jonas Malaco and contributors
+Copyright (C) 2020–2022  Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
@@ -177,7 +177,7 @@ class Ddr4Temperature(SmbusDriver):
             # accidental attempts of writes to the SPD EEPROM (DDR4 SPD writes
             # are also disabled by default in many motherboards)
             dev = cls(smbus, desc, address=(None, None, spd_addr))
-            _LOGGER.debug('instanced driver for %s', desc)
+            _LOGGER.debug('found %s: %s', cls.__name__, desc)
             yield dev
 
     @classmethod
@@ -210,7 +210,7 @@ class Ddr4Temperature(SmbusDriver):
         """
 
         if not check_unsafe(*self._UNSAFE, **kwargs):
-            _LOGGER.warning("%s: nothing returned, requires unsafe features '%s'",
+            _LOGGER.warning("%s: disabled, requires unsafe features '%s'",
                             self.description, ','.join(self._UNSAFE))
             return []
 
@@ -231,7 +231,19 @@ class Ddr4Temperature(SmbusDriver):
         ]
 
     def _read_temperature_register(self):
-        return self._smbus.read_block_data(self._ts_address, self._REG_TEMPERATURE)
+        # in theory we should first write 0x05 to the pointer register, but
+        # avoid writing to the device, even if that means occasionally reading
+        # garbage; ideally we would check the currently set pointer, but we
+        # have not found any way to do that
+
+        # while JEDEC 21-C 4.1.6 uses the term "block read", it has little to
+        # do with the SMBus Block Read protocol; instead, it is closer to the
+        # SMBus Read Word protocol, except in big endianess
+        treg = self._smbus.read_word_data(self._ts_address, self._REG_TEMPERATURE)
+
+        # swap LSB and MSB before returning: read_word_data reads in little
+        # endianess, but the register must be read in big endianess
+        return ((treg & 0xff) << 8) | (treg >> 8)
 
     def initialize(self, **kwargs):
         """Initialize the device."""
@@ -302,14 +314,6 @@ class VengeanceRgb(Ddr4Temperature):
 
         return 'Corsair Vengeance RGB'
 
-    def _read_temperature_register(self):
-        # instead of using block reads, Vengeance RGB temperature sensor
-        # devices must be read in words
-        treg = self._smbus.read_word_data(self._ts_address, self._REG_TEMPERATURE)
-
-        # swap LSB and MSB before returning: read_word_data reads in little
-        # endianess, but the register should be read in big endianess
-        return ((treg & 0xff) << 8) | (treg >> 8)
 
     def set_color(self, channel, mode, colors, speed='normal',
                   transition_ticks=None, stable_ticks=None, **kwargs):
