@@ -2,7 +2,7 @@
 
 For now, these are unstable APIs, and only Linux is supported.
 
-Copyright (C) 2020–2021  Jonas Malaco and contributors
+Copyright (C) 2020–2022  Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
@@ -48,22 +48,26 @@ if sys.platform == 'linux':
                 return
 
             drivers = sorted(find_all_subclasses(SmbusDriver),
-                             key=lambda x: (x.__module__, x.__name__))
+                             key=lambda x: x.__name__)
 
-            _LOGGER.debug('searching %s (%s)', self.__class__.__name__,
-                          ', '.join(map(lambda x: x.__name__, drivers)))
+            _LOGGER.debug('searching %s', self.__class__.__name__)
+            _LOGGER.debug(
+                '%s drivers: %s',
+                self.__class__.__name__,
+                ', '.join(map(lambda x: x.__name__, drivers))
+            )
 
             for i2c_dev in devices.iterdir():
                 try:
                     i2c_bus = LinuxI2cBus(i2c_dev)
                 except ValueError as err:
-                    _LOGGER.debug('ignoring %s, %s', i2c_dev.name, err)
+                    _LOGGER.debug('I²C adapter: %s (skipped, %s)', i2c_dev.name, err)
                     continue
 
                 if bus and bus != i2c_bus.name:
                     continue
 
-                _LOGGER.debug('found I²C bus %s', i2c_bus.name)
+                _LOGGER.debug('I²C adapter: %s', i2c_bus.name)
                 yield from i2c_bus.find_devices(drivers, **kwargs)
 
     class LinuxI2cBus:
@@ -80,14 +84,12 @@ if sys.platform == 'linux':
         # find_liquidctl_devices to try to directly instantiate it
 
         def __init__(self, i2c_dev):
+            if not i2c_dev.name.startswith('i2c-'):
+                raise ValueError('not a bus or unsupported adapter')
+
             self._i2c_dev = i2c_dev
             self._smbus = None
-
-            try:
-                assert i2c_dev.name.startswith('i2c-')
-                self._number = int(i2c_dev.name[4:])
-            except:
-                raise ValueError('cannot infer bus number')
+            self._number = int(i2c_dev.name[4:])
 
         def find_devices(self, drivers, **kwargs):
             """Probe drivers and find compatible devices in this bus."""
@@ -171,7 +173,7 @@ if sys.platform == 'linux':
                 name = self._i2c_dev.joinpath(dev, 'name').read_text().strip()
                 eeprom = self._i2c_dev.joinpath(dev, 'eeprom').read_bytes()
                 return LinuxEeprom(name, eeprom)
-            except Exception:
+            except FileNotFoundError:
                 return None
 
         @property
@@ -284,7 +286,7 @@ class SmbusDriver(BaseDriver):
             # for the 'smbus' feature themselves and, if necessary, raise
             # UnsafeFeaturesNotEnabled(*requirements)
             # (see also: check_unsafe(..., error=True))
-            _LOGGER.debug("SMBus is disabled, missing unsafe feature 'smbus'")
+            _LOGGER.debug("SMBus disabled, missing unsafe feature 'smbus'")
         return self
 
     def disconnect(self, **kwargs):
