@@ -51,7 +51,7 @@ The subclass constructor can generally be kept unaware of the implementation
 details of the device parameter, and find_supported_devices already accepts
 keyword arguments and forwards them to the driver constructor.
 
-Copyright (C) 2019–2021  Jonas Malaco and contributors
+Copyright (C) 2019–2022  Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
@@ -66,8 +66,13 @@ try:
     import hidraw as hid
 except ModuleNotFoundError:
     import hid
+try:
+    import libusb_package
+except ModuleNotFoundError:
+    libusb_package = None
 
 from liquidctl.driver.base import BaseDriver, BaseBus, find_all_subclasses
+from liquidctl.driver.hwmon import HwmonDevice
 from liquidctl.util import LazyHexRepr
 
 _LOGGER = logging.getLogger(__name__)
@@ -92,7 +97,7 @@ class BaseUsbDriver(BaseDriver):
     def probe(cls, handle, vendor=None, product=None, release=None,
               serial=None, match=None, **kwargs):
         """Probe `handle` and yield corresponding driver instances."""
-        for vid, pid, _, description, devargs in cls.SUPPORTED_DEVICES:
+        for vid, pid, _, desc, devargs in cls.SUPPORTED_DEVICES:
             if (vendor and vendor != vid) or handle.vendor_id != vid:
                 continue
             if (product and product != pid) or handle.product_id != pid:
@@ -101,12 +106,12 @@ class BaseUsbDriver(BaseDriver):
                 continue
             if serial and handle.serial_number != serial:
                 continue
-            if match and match.lower() not in description.lower():
+            if match and match.lower() not in desc.lower():
                 continue
             consargs = devargs.copy()
             consargs.update(kwargs)
-            dev = cls(handle, description, **consargs)
-            _LOGGER.debug('instanced driver for %s', description)
+            dev = cls(handle, desc, **consargs)
+            _LOGGER.debug('found %s: %s', cls.__name__, desc)
             yield dev
 
     def __init__(self, device, description, **kwargs):
@@ -196,6 +201,9 @@ class UsbHidDriver(BaseUsbDriver):
             assert hidinfo, 'Could not find device in HID bus'
             device = HidapiDevice(hid, hidinfo)
         super().__init__(device, description, **kwargs)
+        self._hwmon = HwmonDevice.from_hidraw(device.path)
+        if self._hwmon:
+            _LOGGER.debug('has kernel driver: %s (%s)', self._hwmon.module, self._hwmon.path)
 
 
 class UsbDriver(BaseUsbDriver):
@@ -317,7 +325,12 @@ class PyUsbDevice:
             args['idVendor'] = vid
         if pid:
             args['idProduct'] = pid
-        for handle in usb.core.find(find_all=True, **args):
+        if libusb_package and (sys.platform == 'win32' or sys.platform == 'cygwin'):
+            _LOGGER.debug('using libusb_package.find')
+            find = libusb_package.find
+        else:
+            find = usb.core.find
+        for handle in find(find_all=True, **args):
             yield cls(handle)
 
     @property
@@ -482,6 +495,10 @@ class HidapiDevice:
         for info in infos:
             yield cls(api, info)
 
+    @property
+    def path(self):
+        return self.hidinfo['path']
+
     @property
     def vendor_id(self):
         return self.hidinfo['vendor_id']
@@ -519,10 +536,13 @@ class HidapiBus(BaseBus):
                      usb_port=None, **kwargs):
         """Find compatible HID devices."""
         handles = HidapiDevice.enumerate(hid, vendor, product)
-        drivers = sorted(find_all_subclasses(UsbHidDriver),
-                         key=lambda x: (x.__module__, x.__name__))
-        _LOGGER.debug('searching %s (%s)', self.__class__.__name__,
-                      ', '.join(map(lambda x: x.__name__, drivers)))
+        drivers = sorted(find_all_subclasses(UsbHidDriver), key=lambda x: x.__name__)
+        _LOGGER.debug('searching %s', self.__class__.__name__)
+        _LOGGER.debug(
+            '%s drivers: %s',
+            self.__class__.__name__,
+            ', '.join(map(lambda x: x.__name__, drivers))
+        )
         for handle in handles:
             if bus and handle.bus != bus:
                 continue
@@ -530,8 +550,24 @@ class HidapiBus(BaseBus):
                 continue
             if usb_port and handle.port != usb_port:
                 continue
-            _LOGGER.debug('found HID device %04x:%04x', handle.vendor_id,
-                          handle.product_id)
+            # each handle is a HIDAPI hid_device, and that can either mean one
+            # entire HID interface, or one interface ⨯ usage page ⨯ usage id
+            # product, depending on the platform and backend; but, for brevity,
+            # refer them simply as "HID devices"
+            if 'usage' in handle.hidinfo and 'usage_page' in handle.hidinfo:
+                _LOGGER.debug(
+                    'HID device: %04x:%04x (usage_page=%#06x usage=%#06x)',
+                    handle.vendor_id,
+                    handle.product_id,
+                    handle.hidinfo['usage_page'],
+                    handle.hidinfo['usage'],
+                )
+            else:
+                _LOGGER.debug(
+                    'HID device: %04x:%04x (usage n/a)',
+                    handle.vendor_id,
+                    handle.product_id,
+                )
             for drv in drivers:
                 yield from drv.probe(handle, vendor=vendor, product=product, **kwargs)
 
@@ -540,10 +576,13 @@ class PyUsbBus(BaseBus):
     def find_devices(self, vendor=None, product=None, bus=None, address=None,
                      usb_port=None, **kwargs):
         """ Find compatible regular USB devices."""
-        drivers = sorted(find_all_subclasses(UsbDriver),
-                         key=lambda x: (x.__module__, x.__name__))
-        _LOGGER.debug('searching %s (%s)', self.__class__.__name__,
-                      ', '.join(map(lambda x: x.__name__, drivers)))
+        drivers = sorted(find_all_subclasses(UsbDriver), key=lambda x: x.__name__)
+        _LOGGER.debug('searching %s', self.__class__.__name__)
+        _LOGGER.debug(
+            '%s drivers: %s',
+            self.__class__.__name__,
+            ', '.join(map(lambda x: x.__name__, drivers))
+        )
         for handle in PyUsbDevice.enumerate(vendor, product):
             if bus and handle.bus != bus:
                 continue
@@ -551,7 +590,6 @@ class PyUsbBus(BaseBus):
                 continue
             if usb_port and handle.port != usb_port:
                 continue
-            _LOGGER.debug('found USB device %04x:%04x', handle.vendor_id,
-                          handle.product_id)
+            _LOGGER.debug('USB device: %04x:%04x', handle.vendor_id, handle.product_id)
             for drv in drivers:
                 yield from drv.probe(handle, vendor=vendor, product=product, **kwargs)
